#version 430 core


layout (location = 0) out vec4 Color;
layout (location = 1) out vec4 BrightColor;

in VertexData {
	vec3 vs_Pos;
	vec3 vs_Normals;
	vec2 vs_UVs;
	vec4 vs_ShadowCoord;
} vert;


uniform vec3 CameraPos;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
	vec3 position;
} dirL;



struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
};

uniform PointLight pointL[10];

uniform int size;

uniform sampler2D texture_diffuse1; //0
uniform sampler2D texture_specular1; //1
uniform sampler2D shadowMap; //2
uniform float brightness;
uniform bool glow; 
uniform bool buff;

int alpha = 10;
float ks = 0.0f;
const float levels = 6.0f;




float ShadowCalculation(vec4 ShadowCoord)
{
  
    vec3 projCoords = ShadowCoord.xyz / ShadowCoord.w;
   
    projCoords = projCoords * 0.5 + 0.5;
 
    float closestDepth = texture(shadowMap, projCoords.xy).r; 
 
    float currentDepth = projCoords.z;

	 vec3 normal = normalize(vert.vs_Normals);
	vec3 lightDir = normalize(-dirL.direction);

	float bias = max(0.0001 * (1.0 - dot(normal, lightDir)), 0.00005); 

	float shadow = 0.0;
	vec2 texelSize = 1.0/textureSize(shadowMap, 0);
	for (int x = -1; x <= 1; ++x)
	{
		for(int y = -1; y <= 1; ++y)
		{
			float pcf = texture(shadowMap, projCoords.xy + vec2(x,y)* texelSize).r;
			shadow += currentDepth - bias > pcf  ? 1.0 : 0.0;
		}

	}

	shadow /= 9.0;

	if(projCoords.z > 1.0)
        shadow = 0.0;

    return shadow;
}



void main()
{

	vec3 FinalColor;

	if(!glow)
	{
		
			float shadow = ShadowCalculation(vert.vs_ShadowCoord);

			vec3 ambientFinal = texture(texture_diffuse1, vert.vs_UVs).rgb * 0.1;

		//Diffuse Light for Directional Light
		
			vec3 LightDir = normalize(-dirL.direction);
			float DiffuseD = clamp(dot(LightDir, vert.vs_Normals), 0, 1);
		
			float level = floor(DiffuseD * levels);
			DiffuseD = level / levels;

			vec3 diffuseFinalD = texture(texture_diffuse1, vert.vs_UVs).rgb * DiffuseD * 0.9f * dirL.color;

		//Diffuse + Specular Light for Point Lights

			vec3 totalDiffuseP = vec3(0.0);
			vec3 totalSpecularP = vec3(0.0);
			vec3 PosToLight[10];
			float DistToLight;
			vec3 reflectPoint[10];
			vec3 PosToCamera = normalize(vert.vs_Pos - CameraPos);
			float att;

		for (int i = 0; i < size; i++)
		{
		//Diffuse
			PosToLight[i] = normalize(pointL[i].position - vert.vs_Pos);
			DistToLight = length(pointL[i].position - vert.vs_Pos);
			float att = (pointL[i].attenuation.x + DistToLight * pow(pointL[i].attenuation.y, 2) * pointL[i].attenuation.z); 
			float DiffuseP = clamp(dot(PosToLight[i], vert.vs_Normals), 0 , 1);

			//level = floor(DiffuseP * levels);
			//DiffuseP = level / levels;

			totalDiffuseP = totalDiffuseP + ((texture(texture_diffuse1, vert.vs_UVs).rgb * DiffuseP * 0.9f * pointL[i].color)/att);

		//Specular
			reflectPoint[i] = normalize(reflect(PosToLight[i], normalize(vert.vs_Normals)));
			float specularP = pow(max(dot(PosToCamera, reflectPoint[i]),0), alpha);
			totalSpecularP = totalSpecularP + ((texture(texture_specular1, vert.vs_UVs).rgb * pointL[i].color * specularP * ks)/att);
		
		}

		//Specular Light

		vec3 reflectDirectional = normalize(reflect(LightDir, normalize(vert.vs_Normals)));
		float specularD = pow(max(dot(PosToCamera, reflectDirectional), 0), alpha);
		vec3 specularFinalD = specularD * ks * dirL.color;

		

		
		
		FinalColor = (ambientFinal + (1-shadow) * (diffuseFinalD + specularFinalD) + (totalDiffuseP + totalSpecularP));
    }

	
	if(buff == true)
	{
		BrightColor = vec4(FinalColor, 1.0);
	}

	else
	{
		BrightColor = vec4(0,0,0, 1.0);
	}

	
	if (glow)
	{
		FinalColor = vec3(0.0, 0.6, 0.8);
		BrightColor = vec4(FinalColor, 1.0f);
	}

	Color = vec4(brightness * FinalColor, 1.0f);
	
	
}